//+------------------------------------------------------------------+
//|                                           CalculateProfit_EA.mq5 |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"

//---   ,   
string entry_line="entry";
//---   ,   
string exit_line="exit";
//---  
string arrow_line="direct arrow";
//---   
string text_field="profit info";
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---       
   CheckLevelLines();
   return(0);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---    ,   
   ObjectDelete(0,entry_line);
   ObjectDelete(0,exit_line);
   ObjectDelete(0,arrow_line);
   ObjectDelete(0,text_field);
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
   CheckLevelLines();
//---         
   ShowProfit();
  }
//+------------------------------------------------------------------+
//|   /   1         |
//+------------------------------------------------------------------+
double CalculateProfitOneLot(double entry_price,double exit_price)
  {
//---      
   double profit=0;
   if(!OrderCalcProfit(ORDER_TYPE_BUY,Symbol(),1.0,entry_price,exit_price,profit))
     {
      Print(__FUNCTION__,"     OrderCalcProfit().  ",GetLastError());
     }
//---
   return(profit);
  }
//+------------------------------------------------------------------+
//|                                |
//+------------------------------------------------------------------+
void  OnChartEvent(const int id,         //    
                   const long& lparam,   //    long
                   const double& dparam, //    double
                   const string& sparam  //    string
                   )
  {
//---         
   ShowProfit();
  }
//+------------------------------------------------------------------+
//|                               |
//+------------------------------------------------------------------+
void ShowProfit()
  {
//---  
   static double entry=0;
//---  
   static double exit=0;

//---    
   double curr_entry=NormalizeDouble(ObjectGetDouble(0,entry_line,OBJPROP_PRICE),_Digits);
   double curr_exit=NormalizeDouble(ObjectGetDouble(0,exit_line,OBJPROP_PRICE),_Digits);

//---     
   if(entry!=curr_entry || exit!=curr_exit)
     {
      //---   
      entry=curr_entry;
      exit=curr_exit;
      //---       
      double profit=CalculateProfitOneLot(entry,exit);
      //---    
      SetColorOnProfit(profit);
      //---  
      SetArrowedLine(entry,exit,profit);
      //---       
      SetTextInfo(entry,exit,profit);
     }
  }
//+------------------------------------------------------------------+
//|        -       |
//+------------------------------------------------------------------+
void SetColorOnProfit(double profit)
  {
   if(profit>0)
     {
      //---   
      ObjectSetInteger(0,exit_line,OBJPROP_COLOR,Green);
      ChartRedraw();
     }
   else
     {
      //----   
      ObjectSetInteger(0,exit_line,OBJPROP_COLOR,Red);
      ChartRedraw();
     }
  }
//+------------------------------------------------------------------+
//|                |
//+------------------------------------------------------------------+
void SetArrowedLine(double entry,double exit,double profit)
  {
//---   
   if(ObjectFind(0,arrow_line)<0)
     {
      if(!ObjectCreate(0,arrow_line,OBJ_ARROWED_LINE,0,TimeCurrent(),entry,TimeCurrent(),exit))
         Print("     .  ",GetLastError());
     }
//---    
   ObjectSetInteger(0,arrow_line,OBJPROP_TIME,0,TimeCurrent());
   ObjectSetInteger(0,arrow_line,OBJPROP_TIME,1,TimeCurrent());
//---    
   ObjectSetDouble(0,arrow_line,OBJPROP_PRICE,0,entry);
   ObjectSetDouble(0,arrow_line,OBJPROP_PRICE,1,exit);
//---   
   if(profit>0)
      ObjectSetInteger(0,arrow_line,OBJPROP_COLOR,Green);
   else
      ObjectSetInteger(0,arrow_line,OBJPROP_COLOR,Red);
//---    
   ChartRedraw();
  }
//+------------------------------------------------------------------+
//|                                 |
//+------------------------------------------------------------------+
void SetTextInfo(double entry,double exit,double profit)
  {
//---   
   if(ObjectFind(0,text_field)<0)
     {
      if(!ObjectCreate(0,text_field,OBJ_TEXT,0,TimeCurrent()-PeriodSeconds(),(entry+exit)/2))
         Print("    .  ",GetLastError());
     }
   else
     {
      //---   
      ObjectSetInteger(0,text_field,OBJPROP_TIME,0,TimeCurrent()-PeriodSeconds());
      ObjectSetDouble(0,text_field,OBJPROP_PRICE,0,(entry+exit)/2);
     }
//---   
   if(profit>0)
      ObjectSetInteger(0,text_field,OBJPROP_COLOR,Green);
   else
      ObjectSetInteger(0,text_field,OBJPROP_COLOR,Red);
//---   
   ObjectSetInteger(0,text_field,OBJPROP_ANCHOR,ANCHOR_RIGHT);
//---  
   string info=StringFormat("%s: Volume=%.1f lots, Delta=%G %s",Symbol(),1.0,profit,AccountInfoString(ACCOUNT_CURRENCY));
//---    
   ObjectSetString(0,text_field,OBJPROP_TEXT,info);
//---    
   ChartRedraw();
  }
//+------------------------------------------------------------------+
//|     ,                           |
//+------------------------------------------------------------------+
void CheckLevelLines()
  {
//---    
   double maxPrice=0;
//---    
   double minPrice=0;
//---  ,      
   double entry_level;
//---  ,      
   double profit_level;

//---   
   bool entry_line_exist=(ObjectFind(0,entry_line)>=0);
   bool exit_line_exist=(ObjectFind(0,exit_line)>=0);

//---      -  
   if(!entry_line_exist || !exit_line_exist)
     {
      //---    
      maxPrice=ChartGetDouble(0,CHART_PRICE_MAX);
      minPrice=ChartGetDouble(0,CHART_PRICE_MIN);
      //---          
      entry_level=NormalizeDouble((maxPrice+minPrice)/2,_Digits);
      //---        
      profit_level=NormalizeDouble(entry_level-(entry_level-minPrice)/2,_Digits);

      //---   
      if(!entry_line_exist)
        {
         ResetLastError();
         if(!ObjectCreate(0,entry_line,OBJ_HLINE,0,0,entry_level))
            Print("       ",entry_line,".  ",GetLastError());
         else
           {
            //---     
            ObjectSetInteger(0,entry_line,OBJPROP_SELECTABLE,true);
            //---   ,      
            ObjectSetInteger(0,entry_line,OBJPROP_SELECTED,true);
            //---    
            ObjectSetInteger(0,entry_line,OBJPROP_COLOR,Brown);
           }
        }
      if(!exit_line_exist)
        {
         ResetLastError();
         if(!ObjectCreate(0,exit_line,OBJ_HLINE,0,0,profit_level))
            Print("       ",exit_line,".  ",GetLastError());
         else
           {
            //---     
            ObjectSetInteger(0,exit_line,OBJPROP_SELECTABLE,true);
            //---   ,      
            ObjectSetInteger(0,exit_line,OBJPROP_SELECTED,true);
            //---    
            ObjectSetInteger(0,exit_line,OBJPROP_COLOR,Red);
           }
        }
     }
//---
  }
//+------------------------------------------------------------------+
